\version "2.24.1"

#(set-global-staff-size 20)

\include "header.ly"

#(define instrumentname '((Oboe.1   . "Oboe I")
                          (Oboe.2   . "Oboe II")
                          (Bassoon  . "Bassoon")
                          (Horn.1   . "Horn I")
                          (Horn.2   . "Horn II")
                          (Violin.1 . "Violin I")
                          (Violin.2 . "Violin II")
                          (Viola    . "Viola")
                          (CB       . "Cello, Bass")))

#(define cbasso  (ly:make-pitch -1 0 0))
#(define dbasso  (ly:make-pitch -1 1 0))
#(define e♭basso (ly:make-pitch -1 2 -1/2))
#(define ebasso  (ly:make-pitch -1 2 0))
#(define fbasso  (ly:make-pitch -1 3 0))
#(define f♯basso (ly:make-pitch -1 3 1/2))
#(define gbasso  (ly:make-pitch -1 4 0))
#(define aalto   (ly:make-pitch -1 5 0))
#(define b♭alto  (ly:make-pitch -1 6 -1/2))
#(define balto   (ly:make-pitch -1 6 0))
#(define calto   (ly:make-pitch  1 0 0))

#(define (tacet instrument)
  (for-each
    (lambda (movement)
      (let* ((mvt   (ly:parser-lookup movement))
             (music (ly:assoc-get instrument mvt #f))
             (tacet (ly:assoc-get 'tacet mvt)))
        (when (not music)
          (ly:parser-define! movement (assoc-set! mvt instrument tacet)))))
    mvts))

#(define (tacetgroup instrument n)
  (for-each
    (lambda (movement)
      (let* ((mvt    (ly:parser-lookup movement))
             (group  (ly:assoc-get instrument mvt '()))
             (music  (if (not (null? group)) (ly:assoc-get n group #f) #f))
             (tacet  (ly:assoc-get 'tacet mvt)))
        (when (not music)
          (let ((newgroup (assoc-set! group n tacet)))
            (ly:parser-define! movement (assoc-set! mvt instrument newgroup))))))
    mvts))

#(define (mergemusic src dst mvts)
  (for-each
    (lambda (movement)
      (let* ((mvt      (ly:parser-lookup movement))
             (dstmusic (ly:assoc-get dst mvt #f)))
        (when (not dstmusic) ; do not overwrite existing music
          (ly:parser-define! movement (assoc-set! mvt dst (ly:assoc-get src mvt))))))
    mvts))

\bookOutputName #(format #f "~3,'0d-parts" N)

\layout {
  indent = #(* 16 staffspace)
  \compressEmptyMeasures
  \context { \Score barNumberVisibility = #first-bar-number-invisible
             \override VerticalAxisGroup.remove-empty = ##f }
  \context { \StaffGroup \override SystemStartBracket.collapse-height = 5 }
  \context { \Violin instrumentName = "Violin" }
  \context { \Bassoon aDueText = \markup \whiteout "Tutti" soloText = \markup \whiteout "Vlc" }
}

\paper {
  ragged-right = ##f
  ragged-last-bottom = ##f
  page-breaking = #ly:page-turn-breaking
  oddHeaderMarkup = \markup \fill-line {
    \null
    \unless \on-first-page-of-part \normalsize \line { \fromproperty #'header:instrument }
    \if \should-print-page-number \teeny \fromproperty #'page:page-number-string
  }
  evenHeaderMarkup = \markup \fill-line {
    \if \should-print-page-number \teeny \fromproperty #'page:page-number-string
    \unless \on-first-page-of-part \normalsize \line { \fromproperty #'header:instrument }
    \null
  }
  bookTitleMarkup = \markup {
    \override #'(baseline-skip . 4.5)
    \column {
      \override #'(baseline-skip . 3.5)
      \column {
        \fill-line { \huge \larger \larger \bold \fromproperty #'header:title }
        \fill-line { \huge \bold \fromproperty #'header:instrument }
      }
      \fill-line { \null \fromproperty #'header:composer }
    }
  }
}

parttemplate = #(define-scheme-function (movement part) (symbol? symbol?)
  (let* ((ident  (format #f "~a" (symbol->string movement)))
         (piece  (cond ((string-any (char-set #\a) ident) (string-delete (char-set #\a) ident))
                       ((string-any (char-set #\b #\c) ident) "")
                       (else ident)))
         (mvt    (ly:parser-lookup movement))
         (struct (ly:assoc-get 'structure mvt))
         (inst   (string-split (symbol->string part) #\.))
         (ctx    (string->symbol (first inst)))
         (tpose  (ly:assoc-get movement (ly:assoc-get part transpose '()) (ly:make-pitch 0 0 0)))
         (music  (ly:assoc-get ctx mvt))
         (voice  (if (= 2 (length inst))
                   (ly:assoc-get (string->number (second inst)) music)
                   music)))
    #{
      \score {
        <<
          \new Structure { #struct }
          \new #ctx \with { shortInstrumentName = ##f key = #tpose } { #voice }
        >>
        \layout { \breaklayout #movement }
        \header { piece = #piece }
      }
    #}
  ))

#(let* ((filename (ly:parser-output-name))
        (book     (ly:make-book $defaultpaper $defaultheader)))

  ; insert cover page
  (ly:book-add-bookpart! book (cover))

  ; collect all Cello/Bass music in CB
  (mergemusic 'CBB 'CB mvts)

  ; copy movements where Bassoon is playing to Bassoon
  (mergemusic 'CBB 'Bassoon mvts)

  ; when instrument not playing for whole movement, add tacet
  (for-each 
    (lambda (instrument)
      (let* ((inst (string-split (symbol->string instrument) #\.)))
        (if (= 2 (length inst))
          (tacetgroup (string->symbol (first inst)) (string->number (second inst)))
          (tacet (string->symbol (first inst))))))
    orchestra)

  ; assemble music, one bookpart per instrument
  (for-each 
    (lambda (instrument)
      (let* ((part     (map parttemplate mvts (make-list (length mvts) instrument)))
             (bookpart (ly:make-book-part (reverse part))))
        (module-define! $defaultheader 'instrument (assoc-ref instrumentname instrument))
        (ly:book-add-bookpart! book bookpart)))
    orchestra)

  ; process scores with LilyPond
  (ly:book-process book $defaultpaper $defaultlayout filename))
